//**************************************************************************//
//* FRIDA: fast reliable interactive data analysis                         *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                *//
//* http://apps.jcns.fz-juelich.de/frida                                   *//
//**************************************************************************//

//! \file  calc.cpp
//! \brief NCalc::Calculator: the command-line pocket calculator

#include <string>
#include <vector>

#include "defs.h"
#include "olf.h"
#include "mem.h"
#include "reg.h"
#include "var.h"
#include "expr.h"
#include "calc.h"

#include "xax_lex.h"

void NCalc::Calculator( string s ) 
{
    PNode T = user_xaxparse( s.c_str() );
    if( T->has_dummy() )
        throw "dummy argument t not allowed outside curve definitions";
    double ret, dret;
    NOlm::IterateO fiter( NOlm::IterTOL );
    if( fiter.size()==0 || !T->k_dependent() ) {
        T->tree_point_val( &ret, &dret );
        printf("%.12g", ret);
        if( dret )
            printf(" +- %.12g", dret);
        printf( "\n" );
    } else {
        while ( (fiter()) ) {
            int k = fiter.k();
            T->tree_point_val( &ret, &dret, k );
            if ( fiter.size()>1 )
                printf( "f%d: ", k );
            printf("%.12g", ret);
            if( dret )
                printf(" +- %.12g", dret);
            printf( "\n" );
        }
    }
    NReg::lastresult = ret;
}
